/*
 * Copyright (c) 2018 WIT Software. All rights reserved.
 *
 * WIT Software Confidential and Proprietary information. It is strictly forbidden for 3rd parties
 * to modify, decompile,
 * disassemble, defeat, disable or circumvent any protection mechanism; to sell, license, lease,
 * rent, redistribute or
 * make accessible to any third party, whether for profit or without charge.
 */

package com.witsoftware.wmc.login.utils

import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Build
import android.provider.Settings
import com.wit.wcl.Session.SessionRegistrationError
import com.witsoftware.wmc.application.WmcApplication
import com.witsoftware.wmc.appstate.AppStateManager
import com.jio.join.R
import com.witsoftware.wmc.dialogs.DialogManager
import com.witsoftware.wmc.dialogs.DialogParams
import com.witsoftware.wmc.dialogs.utils.DialogValues.*
import com.witsoftware.wmc.location.LocationManager
import com.witsoftware.wmc.location.LocationValues.GpsStatus
import com.witsoftware.wmc.logs.Lucy
import com.witsoftware.wmc.network.NetworkObserver
import com.witsoftware.wmc.network.cb.IConnectivityListener.NetworkType
import com.witsoftware.wmc.notifications.NotificationChannelUtils
import com.witsoftware.wmc.notifications.NotificationManager
import com.witsoftware.wmc.notifications.NotificationUtils
import com.witsoftware.wmc.notifications.NotificationValues.NotificationChannelType
import com.witsoftware.wmc.notifications.NotificationValues.Type
import com.witsoftware.wmc.notifications.types.Notification
import com.witsoftware.wmc.notifications.types.Notification.AlertType
import com.witsoftware.wmc.resources.string
import com.witsoftware.wmc.themes.AttributeManager
import com.witsoftware.wmc.threads.utils.doInBackground
import com.witsoftware.wmc.utils.CompatibilityUtils
import com.witsoftware.wmc.utils.IntentCreator
import com.witsoftware.wmc.utils.Values

class JioLoginHelper private constructor() {

    companion object {

        private const val TAG = "JioLoginHelper"

        @JvmStatic
        fun shouldShowLocationDisabledError(errorCode: SessionRegistrationError): Boolean {
            Lucy.d(TAG, "shouldShowLocationDisabledError | errorCode=$errorCode Android SDK version=${Build.VERSION.SDK_INT}")

            if (errorCode != SessionRegistrationError.REG_ERROR_NETWORK) {
                return false
            }

            if (!CompatibilityUtils.hasMinimumSdk(Build.VERSION_CODES.O)) {
                return false
            }

            val gpsStatus = LocationManager.getInstance().gpsStatus
            val networkType = NetworkObserver.get().networkType
            val cellInfoAvailable = NetworkObserver.get().hasAnyCellInfoAvailable()

            Lucy.d(TAG, "shouldShowLocationDisabledError | gpsStatus=$gpsStatus networkType=$networkType cellInfoAvailable=$cellInfoAvailable")

            return gpsStatus == GpsStatus.OFF && networkType == NetworkType.CELL_JIO && !cellInfoAvailable
        }

        @JvmStatic
        fun createProvisioningErrorLocationDisabledDialog() {
            doInBackground {
                val isAppInForeground = AppStateManager.getInstance().isAppInForeground

                Lucy.d(TAG, "createProvisioningErrorLocationDisabledDialog | isAppInForeground=$isAppInForeground")

                if (!isAppInForeground) {
                    createProvisioningErrorLocationDisableNotification()
                }

                val params = DialogParams.Builder(CustomDialogType.DIALOG_BUTTONS, PriorityLevel.PRIORITY_HIGH)
                        .dialogId(Values.DIALOG_FRAGMENT_PROVISIONING_ERROR_LOCATION_DISABLED)
                        .title(string(R.string.error_code_registration_location_disable_title))
                        .message(string(R.string.error_code_registration_location_disable_message))
                        .persistent(false, DialogContext.PROVISIONING_ERRORS)
                        .addButton(string(R.string.dialog_button_turn_on), ButtonType.BUTTON_POSITIVE) {
                            dismissProvisioningErrorLocationDisabledNotification()

                            val callGPSSettingIntent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
                            callGPSSettingIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                            WmcApplication.getInstance().startActivity(callGPSSettingIntent)
                        }.build()

                DialogManager.get().createDialog(params)
            }
        }

        private fun createProvisioningErrorLocationDisableNotification() {
            val notification = Notification(Type.NOTIFICATION_PROVISIONING_ERRORS,
                    Values.NOTIFICATION_PROVISIONING_ERROR_LOCATION_DISABLED.hashCode(),
                    NotificationChannelUtils.getChannelId(NotificationChannelType.GENERAL))

            notification.smallIconID = AttributeManager.INSTANCE.getAttributeId(R.attr.applicationNotificationIcon)
            notification.ticker = string(R.string.error_code_registration_location_disable_title)
            notification.title = string(R.string.error_code_registration_location_disable_title)
            notification.text = string(R.string.error_code_registration_location_disable_message)
            notification.isAutoCancel = true

            notification.intent = IntentCreator.Base.createSplashScreenIntent(WmcApplication.getContext())
            notification.alertType = AlertType.PLAY_SOUND_AND_VIBRATE

            var bigIconBitmap: Bitmap? = BitmapFactory.decodeResource(WmcApplication.getContext().resources,
                    AttributeManager.INSTANCE.getAttributeId(R.attr.applicationIcon))

            val largeIconDim = NotificationUtils.getLargeIconNotificationSize()
            if (bigIconBitmap != null && largeIconDim[0] > 0 && largeIconDim[1] > 0) {
                bigIconBitmap = Bitmap.createScaledBitmap(bigIconBitmap, largeIconDim[0], largeIconDim[1], true)
            }

            notification.bigIcon = bigIconBitmap
            notification.setShowSmallIcon(false)

            NotificationManager.getInstance().createNotification(notification)
        }

        @JvmStatic
        fun dismissProvisioningErrorLocationDisabled() {
            Lucy.d(TAG, "dismissProvisioningErrorLocationDisabled")

            DialogManager.get().removeDialog(Values.DIALOG_FRAGMENT_PROVISIONING_ERROR_LOCATION_DISABLED)
            dismissProvisioningErrorLocationDisabledNotification()
        }

        private fun dismissProvisioningErrorLocationDisabledNotification() {
            NotificationManager.getInstance().dismissNotification(Type.NOTIFICATION_PROVISIONING_ERRORS,
                    Values.NOTIFICATION_PROVISIONING_ERROR_LOCATION_DISABLED.hashCode())
        }
    }
}